;;; Urcenie dlzky zoznamu
;;;    (op-dlzka '(1 2 3)) -> 3
;;;
;;; Napoved: 
;;;   vseobecny pripad - dlzka zoznamu je o jednotku vacsia ako
;;;                      dlzka cdr zoznamu
;;;   hranicny pripad - zoznam je prazdny

;;; Pocitanie smerom von z rekurzie (pri vynarani)
;;;     d(1 2 3)
;;;     1 + d(2 3)
;;;     1 + 1 + d(3)
;;;     1 + 1 + 1 + d(nil)

(defun op-dlzka (x)
  (if (null x)
      0
      (+ 1 (op-dlzka (cdr x)))))

;;; Pocitanie smerom dovnutra rekurzie (pri vnarani)
;;;     d(1 2 3)
;;;     d(1 2 3)   0
;;;     d(2 3)     1
;;;     d(3)       2
;;;     d(nil)     3

(defun op-dlzka-aux (x acc)
  (if (null x) 
      acc
      (op-dlzka-aux (cdr x) (1+ acc))))

(defun op-dlzka (x)
  (op-dlzka-aux x 0))


;;; Funkcia nthcdr
;;;    (op-nthcdr 1 '(1 2 3)) -> (2 3)
;;;    (op-nthcdr 5 '(1 2 3)) -> nil
;;;
;;; Napoved: 
;;;   vseobecny pripad - n-ty cdr zoznamu je vlastne 
;;;                      (n-1)-ty cdr zoznamu skrateneho o hlavu
;;;   hranicny pripad - n>0, zoznam je prazdny
;;;   hranicny pripad - n=0

(defun op-nthcdr (n x)
  (cond ((null x) nil)
        ((zerop n) x)
        (t (op-nthcdr (1- n) (cdr x)))))


;;; Funkcia remove
;;;    (op-remove 3 '(1 3 2 3 4)) -> (1 2 4)
;;;
;;; Napoved:
;;;   vseobecny pripad - hlavu vypustit a spracovat cdr zoznamu
;;;   vseobecny pripad - hlavu ponechat a spracovat cdr zoznamu
;;;   hranicny pripad - prazdny zoznam

;;; Pocitanie smerom von z rekurzie (pri vynarani)
;;;      r3(1 3 4)
;;;     (1 r3(3 4))
;;;     (1   r3(4))
;;;     (1      4)

(defun op-remove (vzor x)
  (cond ((null x) nil)
        ((eql vzor (car x)) (op-remove vzor (cdr x)))
        (t (cons (car x) (op-remove vzor (cdr x))))))

;;; Pocitanie smerom dovnutra rekurzie (pri vnarani)
;;;     r3(1 3 4)
;;;     r3(1 3 4)   ()
;;;     r3(3 4)    (1)
;;;     r3(4)      (1)
;;;     r3()     (4 1)
;;;              (1 4)

(defun op-remove-aux (vzor x acl)
  (cond ((null x) (reverse acl))
        ((eql vzor (car x)) (op-remove-aux vzor (cdr x) acl))
        (t (op-remove-aux vzor (cdr x) (cons (car x) acl)))))

(defun op-remove (vzor x)
  (op-remove-aux vzor x nil))
